import argparse
from libbananadm import client
from libbananadm import user
from libbananadm import agent
from libbananadm import backend
from libbananadm import monitor
from libbananadm import input


def reconfigure(args):
    monitor.reconfigure(args)


def create_user(args):
    args.client = input.prompt('client in which create the user')
    args.name = input.prompt('username')
    user.create_user(args)


def create_agent(args):
    args.client = input.prompt('client in which create the agent(s)')
    agent.create_agent(args)


def create_client(args):
    args.name = input.prompt('client name')
    client.create_client(args)


def create_backend_secret(args, type):
    args.client = input.prompt('client in which create the secret')
    args.name = input.prompt('backend name')
    values = backend.prompt_secret_values(type)
    backend.create_backend_secret(args, values)


def init_arguments():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(title='subcommands')

    subparsers.add_parser(
        'init',
        help='setup monitor policies'
    ).set_defaults(func=monitor.init)

    subparsers.add_parser(
        'reconfigure',
        help='ask monitor\'s nginx to re-fetch all CA certs'
    ).set_defaults(func=reconfigure)

    parser_list = subparsers.add_parser(
        'list',
        help='list clients|users|agents|backends'
    )
    subparsers_list = parser_list.add_subparsers(
        title='types of resources',
    )

    subparsers_list.add_parser(
        'clients',
        help='list clients stored in vault',
    ).set_defaults(func=client.list_clients)

    list_users = subparsers_list.add_parser(
        'users',
        help='list users for a given client',
    )
    list_users.add_argument('client', help='client name')
    list_users.set_defaults(func=user.list_users)

    list_agents = subparsers_list.add_parser(
        'agents',
        help='list agents for a given client',
    )
    list_agents.add_argument('client', help='client name')
    list_agents.set_defaults(func=agent.list_agents)

    list_backends = subparsers_list.add_parser(
        'backends',
        help='list backends for a given client',
    )
    list_backends.add_argument('client', help='client name')
    list_backends.set_defaults(func=backend.list_backends)

    parser_new = subparsers.add_parser(
        'new',
        help='create a client|user|agent|backend'
    )
    subparsers_new = parser_new.add_subparsers(
        title='types that can be created',
    )

    subparsers_new.add_parser(
        'client',
        help='create a client',
    ).set_defaults(func=create_client)
    subparsers_new.add_parser(
        'user',
        help='create a user',
    ).set_defaults(func=create_user)
    subparsers_new.add_parser(
        'agent',
        help='create a agent',
    ).set_defaults(func=create_agent)

    parser_backend_secret = subparsers_new.add_parser(
        'backend',
        help='add a backend secret',
    )
    subparsers_backend_secret = parser_backend_secret.add_subparsers(
        title='supported backend types',
    )
    subparsers_backend_secret.add_parser(
        's3',
        help='s3 protocol credentials, also works \
            with openstack block storage',
    ).set_defaults(func=lambda args: create_backend_secret(args, 's3'))

    parser.add_argument(
        '--tls-skip-verify',
        help='ignore TLS errors',
        action='store_true'
    )
    parser.add_argument(
        '--root-path',
        help='root vault mount point',
        default='banana',
    )
    parser.add_argument(
        '--monitor-addr',
        help='monitor API address',
        default='https://api.banana.enix.io',
    )

    return parser



import os
from libbananadm import vault
from tabulate import tabulate


def create_agent(args):
    print('generating temporary token to allow new agent(s) to register')
    client = vault.get_vault_client(args)
    token = client.create_token(
        policies=['{}-{}-agent-creation'.format(args.root_path, args.client)],
        lease='1h',
    )

    print('success! join your new agent(s) using:\n')
    print(
        'bananactl {} --vault-addr={} init {} {} <agent name>'
        .format(
            '--tls-skip-verify' if args.tls_skip_verify else '',
            os.getenv('VAULT_ADDR'),
            token['auth']['client_token'],
            args.client,
        )
    )


def list_agents(args):
    mount_point = '{}/{}/agents-pki'.format(args.root_path, args.client)
    users = filter(
        lambda x: x[0] != '{} Agent Intermediate CA'.format(args.client),
        vault.list_cn_from_pki(args, mount_point),
    )

    print(tabulate(users, headers=['Name', 'Serial']))



import os
import requests
from libbananadm import vault
from libbananadm import policies
from libbananadm import monitor
from tabulate import tabulate
from hvac import exceptions


def create_client(args):
    client = vault.get_vault_client(args)
    client_pki = '{}/{}/root-pki'.format(args.root_path, args.name)
    client_kv = '{}/{}/secrets'.format(args.root_path, args.name)
    client_users_pki = '{}/{}/users-pki'.format(args.root_path, args.name)
    client_agents_pki = '{}/{}/agents-pki'.format(args.root_path, args.name)

    try:
        client.sys.enable_secrets_engine('kv-v2', path=client_kv)
        print('mounted KV at path \'{}\''.format(client_kv))
    except exceptions.InvalidRequest as error:
        if 'existing mount' in error.errors[0]:
            print('error: client {} already exists'.format(args.name))
            exit(1)
        raise error

    client.sys.enable_secrets_engine('pki', path=client_pki, config={
        'max_lease_ttl': '43800h',
    })
    client.secrets.pki.generate_root(
        type='internal',
        common_name=args.name,
        mount_point=client_pki,
        extra_params={
            'ttl': '43800h',
        },
    )
    print('mounted root PKI at path \'{}\''.format(client_pki))

    vault.create_intermediate_ca(
        args, client_pki, client_users_pki, 'User',
    )
    print(
        'mounted users intermediate PKI at path \'{}\''
        .format(client_users_pki)
    )

    agents_cert = vault.create_intermediate_ca(
        args, client_pki, client_agents_pki, 'Agent',
    )
    print(
        'mounted agents intermediate PKI at path \'{}\''
        .format(client_agents_pki)
    )

    create_policy_name = '{}-{}-agent-creation'.format(
        args.root_path,
        args.name,
    )
    client.sys.create_or_update_policy(
        name=create_policy_name,
        policy=policies.generate_agent_install_policy(args),
    )
    print('created policy {}'.format(create_policy_name))

    access_policy_name = '{}-{}-agent-access'.format(args.root_path, args.name)
    client.sys.create_or_update_policy(
        name=access_policy_name,
        policy=policies.generate_agent_access_policy(args),
    )
    print('created policy {}'.format(access_policy_name))

    res = requests.post(
        '{}/v1/auth/banana/cert/certs/{}'.format(
            os.getenv('VAULT_ADDR'),
            args.name,
        ),
        json={
            'display_name': args.name,
            'policies': '{}-{}-agent-access'.format(args.root_path, args.name),
            'certificate': agents_cert,
        },
        headers={
            'X-Vault-Token': os.getenv('VAULT_TOKEN'),
        },
        verify=not args.tls_skip_verify,
    )
    if res.status_code >= 400:
        print(res.json())
        exit(1)
    print('allowed agent certs to login into vault')

    monitor.reconfigure(args)
    print('successfully created client \'{}\''.format(args.name))


def list_clients(args):
    client = vault.get_vault_client(args)
    secrets_engines = client.sys.list_mounted_secrets_engines()
    output = []

    for key in secrets_engines['data']:
        parts = key.split('/')
        if parts[0] != 'banana' or parts[2] != 'root-pki':
            continue
        output.append([parts[1]])

    print(tabulate(output, headers=['Name']))



def prompt(question, default=None):
    if default is None:
        default_text = ''
    else:
        default_text = ' [{0}]'.format(default)
    question = '{0}{1}? '.format(question, default_text)
    answer = input(question).rstrip('\n')
    if not answer:
        answer = default
    return answer



#! /usr/bin/env python3

import os
import setuptools

setuptools.setup(
    name='bananadm',
    version=os.getenv('CI_COMMIT_TAG'),
    author='Arthur Chaloin from Enix <enix.io>',
    author_email='arthur.chaloin@enix.fr',
    description='A command line tool to manage Banana instances',
    packages=setuptools.find_packages(),
    scripts=['bananadm'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'urllib3',
        'requests>=2.21.0',
        'hvac',
        'cryptography',
        'tabulate',
    ],
)



import hvac
import os
from cryptography import x509, hazmat


def get_vault_client(args):
    verify = False if args.tls_skip_verify else True
    vault = hvac.Client(
        url=os.getenv('VAULT_ADDR'),
        token=os.getenv('VAULT_TOKEN'),
        verify=verify,
    )
    return vault


def create_cert(args, cn, pki, role):
    vault = get_vault_client(args)
    res = vault.secrets.pki.generate_certificate(
        name=role,
        common_name=cn,
        mount_point=pki,
    )
    cert = res.json()['data']['certificate']
    key = res.json()['data']['private_key']
    return cert, key


def create_intermediate_ca(args, root_path, int_path, type):
    vault = get_vault_client(args)
    vault.sys.enable_secrets_engine('pki', path=int_path, config={
        'max_lease_ttl': '43800h',
    })
    users_int = vault.secrets.pki.generate_intermediate(
        type='internal',
        common_name='{} {} Intermediate CA'.format(args.name, type),
        mount_point=int_path,
        extra_params={
            'ttl': '43800h',
        },
    )
    signed_users_int = vault.secrets.pki.sign_intermediate(
        csr=users_int.json()['data']['csr'],
        common_name='{} {} Intermediate CA'.format(args.name, type),
        mount_point=root_path,
        extra_params={
            'ttl': '43800h',
        },
    )
    int_cert = signed_users_int.json()['data']['certificate']
    vault.secrets.pki.set_signed_intermediate(
        int_cert,
        mount_point=int_path,
    )
    vault.secrets.pki.create_or_update_role('default', {
        'allow_any_name': 'true',
        'organization': args.name,
        'ou': type,
        'default_lease_ttl': '17520h',
    }, mount_point=int_path)
    return int_cert


def list_cn_from_pki(args, mount_point):
    client = get_vault_client(args)
    response = client.secrets.pki.list_certificates(
        mount_point=mount_point,
    )
    output = []

    for key in response['data']['keys']:
        cert_data = client.secrets.pki.read_certificate(
            serial=key,
            mount_point=mount_point,
        )

        cert_str = cert_data['data']['certificate']
        cert = x509.load_pem_x509_certificate(
            cert_str.encode('ascii'),
            hazmat.backends.default_backend(),
        )

        name = cert.subject.get_attributes_for_oid(
            x509.NameOID.COMMON_NAME,
        )[0].value

        output.append([name, key])

    return output
